// -*-C++-*-
/*!
 * @file  LRFInfoSVC_impl.cpp
 * @brief Service implementation code of LRFInfo.idl
 *
 */

#include "LRFInfoSVC_impl.h"
#include "sick.h"
#include <cmath>

#define DEG2RAD(x) ((x)*M_PI/180.0)
/*
 * Example implementational code for IDL interface LRFInfo
 */
LRFInfoSVC_impl::LRFInfoSVC_impl(sick_laser_p laser)
{
  m_laser = laser;
  // Please add extra constructor code here.
}


LRFInfoSVC_impl::~LRFInfoSVC_impl()
{
  // Please add extra destructor code here.
}


/*
 * Methods corresponding to IDL attributes and operations
 */
CORBA::Short LRFInfoSVC_impl::getTotalScanPoints()
{
  // データ点数を返す
  return m_laser->numvalues;
}

CORBA::Double LRFInfoSVC_impl::getResolution()
{
  // RES_1_00_DEGREE（1.0deg）モードの時は 1.0deg の rad値を
  // RES_0_50_DEGREE（0.5deg）モードの時は 0.5deg の rad値を
  // それ以外では 0.25deg の rad値を返す
  return (m_laser->settings.angle_resolution == RES_1_00_DEGREE ? DEG2RAD(1.0) :
	  m_laser->settings.angle_resolution == RES_0_50_DEGREE ? DEG2RAD(0.5) :
	  DEG2RAD(0.25) );
}

CORBA::Double LRFInfoSVC_impl::getOffsetFOV()
{
  // 0時を正面として3時方向を0radとするオフセット量を出力．反時計回りを正とする．
  // 180degモード時には
  return (DEG2RAD(90-m_laser->settings.angle_range/2));
}



// End of example implementational code



