/* 

 Copyright(C) 2006 Osamu Matsumoto <osamu-m@is.naist.jp>

 This is free software with ABSOLUTELY NO WARRANTY.
 You can redistribute it and/or modify it under the terms of
 the GNU General Public License version 2.

 Original Source code is part of CARMEN(Carnegie Mellon Robot 
 Navigation Toolkit) 

 CARMEN Copyright (c) 2002 Michael Montemerlo, Nicholas
 Roy, Sebastian Thrun, Dirk Haehnel, Cyrill Stachniss,
 and Jared Glover

 2008.11.20  This source code is editted by Tsuyoshi Suenaga, NAIST
             <tsuyo-s@is.naist.jp>

*/
#ifndef CARMEN_SICK_H
#define CARMEN_SICK_H

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <unistd.h>
#include <termios.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <linux/serial.h>
#include <fcntl.h>

#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif


#define LASER_BUFFER_SIZE                100000

#define CRC16_GEN_POL                    0x8005
#define CRC16_GEN_POL0                   0x80
#define CRC16_GEN_POL1                   0x05

#define LMS_PASSWORD                     "SICK_LMS"
#define PLS_PASSWORD                     "SICK_PLS"

#define BUFFER_SIZE                      16000
#define MAX_COMMAND_SIZE                 8196
#define MAX_NAME_LENGTH                  256

#define MAX_TIME_FOR_CLEAR               0.2
#define MAX_TIME_FOR_DATA                0.3
#define MAX_TIME_FOR_ACK                 0.1
#define MAX_TIME_FOR_ANSWER              0.1
#define MAX_TIME_FOR_SENSOR_DATA         0.5
#define MAX_TIME_FOR_CONFIG              3.0
#define MAX_TIME_FOR_GETTING_CONF        0.3
#define MAX_TIME_FOR_TESTING_BAUDRATE    1.0

#define INI                              -1
#define TIO                              0
#define STX                              0x02
#define UKN                              0x05
#define ACK                              0x06
#define DLE                              0x10
#define NAK                              0x15
#define LID                              0x80

#define RES_1_00_DEGREE                  0
#define RES_0_50_DEGREE                  1 
#define RES_0_25_DEGREE                  2

#ifndef TIOCGETP 
#define TIOCGETP                         0x5481
#define TIOCSETP                         0x5482
#define RAW                              1
#define CBREAK                           64
#endif

typedef enum { PLS, LMS } laser_model_t;
typedef enum { CM, MM, DM } range_res_t;
typedef enum { RANGE80M, RANGE160M, RANGE320M, REMISSION_NORM, REMISSION_DIRECT } range_dist_t;
typedef enum { N, E, O } parity_t;

typedef struct {
  int                fd;
  laser_model_t      type;
  char               *ttyport;
  int                baudrate;
  parity_t           parity;
  unsigned char      passwd[8];
  int                databits;
  int                stopbits;
  int                hwf;
  int                swf;
  int                laser_num;
} laser_device_t, *laser_device_p;

typedef struct {
  char device_name[MAX_NAME_LENGTH];
  laser_model_t type;
  range_res_t range_res;
  range_dist_t range_dist;
  unsigned char password[8];
  int laser_num;
  int detect_baudrate, use_highspeed;
  int start_baudrate, set_baudrate;
  int databits, stopbits;
  parity_t parity;
  int swf, hwf;
  int angle_range, angle_resolution;
  int use_remission;
  int rem_values;
  int num_values;
  int laser_flipped;
} laser_settings_t;

typedef struct {
  laser_settings_t settings;
  laser_device_t dev;

  int numvalues;
  double *range;
  int *glare, *wfv, *sfv;
// *** REI - START *** //
  double *remission;
  int remvalues;
// *** REI - END *** //
  unsigned char *buffer;
  long int buffer_position, processed_mark, packet_offset, packet_length;
  int new_reading;
  double timestamp;
  double packet_timestamp;
} sick_laser_t, *sick_laser_p;

int sick_start_laser(sick_laser_p laser);
void sick_stop_laser(sick_laser_p laser);
void sick_handle_laser(sick_laser_p laser);

// for parameter setting
void sick_set_default_parameters(sick_laser_p laser, int laser_num);
int sick_parse_params(sick_laser_p laser, const char *dev, const char *type, 
		      double res, const char *rem, double fov, int speed);

#endif
